# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'HKLviewer.ui',
# licensing of 'HKLviewer.ui' applies.
#
# Created: Tue Mar  3 13:21:17 2020
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!
from __future__ import absolute_import, division, print_function

from PySide2 import QtCore, QtWidgets

class Ui_MainWindow(object):
  def setupUi(self, MainWindow):
    MainWindow.setObjectName("MainWindow")
    MainWindow.resize(1066, 840)
    self.centralwidget = QtWidgets.QWidget(MainWindow)
    self.centralwidget.setObjectName("centralwidget")
    self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
    self.gridLayout_2.setObjectName("gridLayout_2")
    self.widget = QtWidgets.QWidget(self.centralwidget)
    self.widget.setObjectName("widget")
    self.gridLayout = QtWidgets.QGridLayout(self.widget)
    self.gridLayout.setContentsMargins(0, 0, 0, 0)
    self.gridLayout.setObjectName("gridLayout")
    self.splitter = QtWidgets.QSplitter(self.widget)
    self.splitter.setOrientation(QtCore.Qt.Horizontal)
    self.splitter.setObjectName("splitter")
    self.widget_2 = QtWidgets.QWidget(self.splitter)
    self.widget_2.setObjectName("widget_2")
    self.gridLayout_4 = QtWidgets.QGridLayout(self.widget_2)
    self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_4.setObjectName("gridLayout_4")
    self.splitter_2 = QtWidgets.QSplitter(self.widget_2)
    self.splitter_2.setFrameShape(QtWidgets.QFrame.NoFrame)
    self.splitter_2.setFrameShadow(QtWidgets.QFrame.Plain)
    self.splitter_2.setMidLineWidth(0)
    self.splitter_2.setOrientation(QtCore.Qt.Vertical)
    self.splitter_2.setObjectName("splitter_2")
    self.widget_4 = QtWidgets.QWidget(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(3)
    sizePolicy.setHeightForWidth(self.widget_4.sizePolicy().hasHeightForWidth())
    self.widget_4.setSizePolicy(sizePolicy)
    self.widget_4.setObjectName("widget_4")
    self.gridLayout_5 = QtWidgets.QGridLayout(self.widget_4)
    self.gridLayout_5.setContentsMargins(0, 6, 0, -1)
    self.gridLayout_5.setObjectName("gridLayout_5")
    self.label = QtWidgets.QLabel(self.widget_4)
    self.label.setObjectName("label")
    self.gridLayout_5.addWidget(self.label, 0, 0, 1, 1)
    self.millertable = HeaderDataTableWidget(self.widget_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.millertable.sizePolicy().hasHeightForWidth())
    self.millertable.setSizePolicy(sizePolicy)
    self.millertable.setRowCount(5)
    self.millertable.setColumnCount(10)
    self.millertable.setObjectName("millertable")
    self.millertable.setColumnCount(10)
    self.millertable.setRowCount(5)
    self.gridLayout_5.addWidget(self.millertable, 1, 0, 1, 1)
    self.functionTabWidget = QtWidgets.QTabWidget(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.functionTabWidget.sizePolicy().hasHeightForWidth())
    self.functionTabWidget.setSizePolicy(sizePolicy)
    self.functionTabWidget.setObjectName("functionTabWidget")
    self.tab = QtWidgets.QWidget()
    self.tab.setObjectName("tab")
    self.gridLayout_28 = QtWidgets.QGridLayout(self.tab)
    self.gridLayout_28.setObjectName("gridLayout_28")
    self.ExpandReflsGroupBox = QtWidgets.QGroupBox(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ExpandReflsGroupBox.sizePolicy().hasHeightForWidth())
    self.ExpandReflsGroupBox.setSizePolicy(sizePolicy)
    self.ExpandReflsGroupBox.setCheckable(True)
    self.ExpandReflsGroupBox.setChecked(False)
    self.ExpandReflsGroupBox.setObjectName("ExpandReflsGroupBox")
    self.gridLayout_3 = QtWidgets.QGridLayout(self.ExpandReflsGroupBox)
    self.gridLayout_3.setContentsMargins(6, 6, 6, 6)
    self.gridLayout_3.setObjectName("gridLayout_3")
    self.expandP1checkbox = QtWidgets.QCheckBox(self.ExpandReflsGroupBox)
    self.expandP1checkbox.setObjectName("expandP1checkbox")
    self.gridLayout_3.addWidget(self.expandP1checkbox, 0, 0, 1, 1)
    self.expandAnomalouscheckbox = QtWidgets.QCheckBox(self.ExpandReflsGroupBox)
    self.expandAnomalouscheckbox.setObjectName("expandAnomalouscheckbox")
    self.gridLayout_3.addWidget(self.expandAnomalouscheckbox, 0, 1, 1, 1)
    self.gridLayout_28.addWidget(self.ExpandReflsGroupBox, 0, 0, 1, 3)
    self.SpacegroupLabel = QtWidgets.QLabel(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.SpacegroupLabel.sizePolicy().hasHeightForWidth())
    self.SpacegroupLabel.setSizePolicy(sizePolicy)
    self.SpacegroupLabel.setObjectName("SpacegroupLabel")
    self.gridLayout_28.addWidget(self.SpacegroupLabel, 2, 0, 1, 1)
    self.sysabsentcheckbox = QtWidgets.QCheckBox(self.tab)
    self.sysabsentcheckbox.setObjectName("sysabsentcheckbox")
    self.gridLayout_28.addWidget(self.sysabsentcheckbox, 1, 0, 1, 1)
    self.widget_3 = QtWidgets.QWidget(self.tab)
    self.widget_3.setObjectName("widget_3")
    self.gridLayout_20 = QtWidgets.QGridLayout(self.widget_3)
    self.gridLayout_20.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_20.setObjectName("gridLayout_20")
    self.missingcheckbox = QtWidgets.QCheckBox(self.widget_3)
    self.missingcheckbox.setObjectName("missingcheckbox")
    self.gridLayout_20.addWidget(self.missingcheckbox, 0, 0, 2, 2)
    self.onlymissingcheckbox = QtWidgets.QCheckBox(self.widget_3)
    self.onlymissingcheckbox.setObjectName("onlymissingcheckbox")
    self.gridLayout_20.addWidget(self.onlymissingcheckbox, 2, 0, 1, 1)
    self.gridLayout_28.addWidget(self.widget_3, 1, 1, 1, 2)
    self.SpaceGroupComboBox = QtWidgets.QComboBox(self.tab)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.SpaceGroupComboBox.sizePolicy().hasHeightForWidth())
    self.SpaceGroupComboBox.setSizePolicy(sizePolicy)
    self.SpaceGroupComboBox.setObjectName("SpaceGroupComboBox")
    self.gridLayout_28.addWidget(self.SpaceGroupComboBox, 2, 1, 1, 2)
    self.functionTabWidget.addTab(self.tab, "")
    self.tab_2 = QtWidgets.QWidget()
    self.tab_2.setObjectName("tab_2")
    self.gridLayout_9 = QtWidgets.QGridLayout(self.tab_2)
    self.gridLayout_9.setObjectName("gridLayout_9")
    self.SliceReflectionsBox = QtWidgets.QCheckBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(10)
    sizePolicy.setHeightForWidth(self.SliceReflectionsBox.sizePolicy().hasHeightForWidth())
    self.SliceReflectionsBox.setSizePolicy(sizePolicy)
    self.SliceReflectionsBox.setObjectName("SliceReflectionsBox")
    self.gridLayout_9.addWidget(self.SliceReflectionsBox, 0, 0, 1, 1)
    self.fixedorientcheckbox = QtWidgets.QCheckBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(3)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.fixedorientcheckbox.sizePolicy().hasHeightForWidth())
    self.fixedorientcheckbox.setSizePolicy(sizePolicy)
    self.fixedorientcheckbox.setObjectName("fixedorientcheckbox")
    self.gridLayout_9.addWidget(self.fixedorientcheckbox, 0, 1, 1, 1)
    self.ClipPlaneChkGroupBox = QtWidgets.QGroupBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.ClipPlaneChkGroupBox.sizePolicy().hasHeightForWidth())
    self.ClipPlaneChkGroupBox.setSizePolicy(sizePolicy)
    self.ClipPlaneChkGroupBox.setCheckable(True)
    self.ClipPlaneChkGroupBox.setObjectName("ClipPlaneChkGroupBox")
    self.gridLayout_7 = QtWidgets.QGridLayout(self.ClipPlaneChkGroupBox)
    self.gridLayout_7.setContentsMargins(3, 3, 3, 6)
    self.gridLayout_7.setObjectName("gridLayout_7")
    self.clipNormalBtn = QtWidgets.QRadioButton(self.ClipPlaneChkGroupBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipNormalBtn.sizePolicy().hasHeightForWidth())
    self.clipNormalBtn.setSizePolicy(sizePolicy)
    self.clipNormalBtn.setObjectName("clipNormalBtn")
    self.gridLayout_7.addWidget(self.clipNormalBtn, 0, 1, 1, 1)
    self.clipParallelBtn = QtWidgets.QRadioButton(self.ClipPlaneChkGroupBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipParallelBtn.sizePolicy().hasHeightForWidth())
    self.clipParallelBtn.setSizePolicy(sizePolicy)
    self.clipParallelBtn.setObjectName("clipParallelBtn")
    self.gridLayout_7.addWidget(self.clipParallelBtn, 0, 0, 1, 1)
    self.groupBox_2 = QtWidgets.QGroupBox(self.ClipPlaneChkGroupBox)
    self.groupBox_2.setObjectName("groupBox_2")
    self.gridLayout_19 = QtWidgets.QGridLayout(self.groupBox_2)
    self.gridLayout_19.setObjectName("gridLayout_19")
    self.frame_2 = QtWidgets.QFrame(self.groupBox_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_2.sizePolicy().hasHeightForWidth())
    self.frame_2.setSizePolicy(sizePolicy)
    self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_2.setObjectName("frame_2")
    self.gridLayout_15 = QtWidgets.QGridLayout(self.frame_2)
    self.gridLayout_15.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_15.setObjectName("gridLayout_15")
    self.recipvecBtn = QtWidgets.QRadioButton(self.frame_2)
    self.recipvecBtn.setObjectName("recipvecBtn")
    self.gridLayout_15.addWidget(self.recipvecBtn, 0, 0, 1, 1)
    self.clipTNCSBtn = QtWidgets.QRadioButton(self.frame_2)
    self.clipTNCSBtn.setObjectName("clipTNCSBtn")
    self.gridLayout_15.addWidget(self.clipTNCSBtn, 0, 1, 1, 1)
    self.realspacevecBtn = QtWidgets.QRadioButton(self.frame_2)
    self.realspacevecBtn.setObjectName("realspacevecBtn")
    self.gridLayout_15.addWidget(self.realspacevecBtn, 1, 0, 1, 1)
    self.gridLayout_19.addWidget(self.frame_2, 0, 0, 1, 1)
    self.frame_6 = QtWidgets.QFrame(self.groupBox_2)
    self.frame_6.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_6.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_6.setObjectName("frame_6")
    self.gridLayout_18 = QtWidgets.QGridLayout(self.frame_6)
    self.gridLayout_18.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_18.setObjectName("gridLayout_18")
    self.rotavecangle_labeltxt = QtWidgets.QLabel(self.frame_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.rotavecangle_labeltxt.sizePolicy().hasHeightForWidth())
    self.rotavecangle_labeltxt.setSizePolicy(sizePolicy)
    self.rotavecangle_labeltxt.setObjectName("rotavecangle_labeltxt")
    self.gridLayout_18.addWidget(self.rotavecangle_labeltxt, 0, 0, 1, 1)
    self.rotavecangle_slider = QtWidgets.QSlider(self.frame_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.rotavecangle_slider.sizePolicy().hasHeightForWidth())
    self.rotavecangle_slider.setSizePolicy(sizePolicy)
    self.rotavecangle_slider.setMaximum(359)
    self.rotavecangle_slider.setOrientation(QtCore.Qt.Horizontal)
    self.rotavecangle_slider.setTickPosition(QtWidgets.QSlider.TicksAbove)
    self.rotavecangle_slider.setTickInterval(18)
    self.rotavecangle_slider.setObjectName("rotavecangle_slider")
    self.gridLayout_18.addWidget(self.rotavecangle_slider, 0, 1, 1, 1)
    self.gridLayout_19.addWidget(self.frame_6, 3, 0, 1, 1)
    self.frame_3 = QtWidgets.QFrame(self.groupBox_2)
    self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_3.setObjectName("frame_3")
    self.gridLayout_16 = QtWidgets.QGridLayout(self.frame_3)
    self.gridLayout_16.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_16.setObjectName("gridLayout_16")
    self.frame_9 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_9.sizePolicy().hasHeightForWidth())
    self.frame_9.setSizePolicy(sizePolicy)
    self.frame_9.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_9.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_9.setObjectName("frame_9")
    self.gridLayout_22 = QtWidgets.QGridLayout(self.frame_9)
    self.gridLayout_22.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_22.setObjectName("gridLayout_22")
    self.label_21 = QtWidgets.QLabel(self.frame_9)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_21.sizePolicy().hasHeightForWidth())
    self.label_21.setSizePolicy(sizePolicy)
    self.label_21.setObjectName("label_21")
    self.gridLayout_22.addWidget(self.label_21, 0, 0, 1, 1)
    self.hvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_9)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.hvec_spinBox.sizePolicy().hasHeightForWidth())
    self.hvec_spinBox.setSizePolicy(sizePolicy)
    self.hvec_spinBox.setObjectName("hvec_spinBox")
    self.gridLayout_22.addWidget(self.hvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_9, 0, 0, 1, 1)
    self.frame_10 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_10.sizePolicy().hasHeightForWidth())
    self.frame_10.setSizePolicy(sizePolicy)
    self.frame_10.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_10.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_10.setObjectName("frame_10")
    self.gridLayout_23 = QtWidgets.QGridLayout(self.frame_10)
    self.gridLayout_23.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_23.setObjectName("gridLayout_23")
    self.label_22 = QtWidgets.QLabel(self.frame_10)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_22.sizePolicy().hasHeightForWidth())
    self.label_22.setSizePolicy(sizePolicy)
    self.label_22.setObjectName("label_22")
    self.gridLayout_23.addWidget(self.label_22, 0, 0, 1, 1)
    self.kvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_10)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.kvec_spinBox.sizePolicy().hasHeightForWidth())
    self.kvec_spinBox.setSizePolicy(sizePolicy)
    self.kvec_spinBox.setObjectName("kvec_spinBox")
    self.gridLayout_23.addWidget(self.kvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_10, 0, 1, 1, 1)
    self.frame_11 = QtWidgets.QFrame(self.frame_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_11.sizePolicy().hasHeightForWidth())
    self.frame_11.setSizePolicy(sizePolicy)
    self.frame_11.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_11.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_11.setObjectName("frame_11")
    self.gridLayout_24 = QtWidgets.QGridLayout(self.frame_11)
    self.gridLayout_24.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_24.setObjectName("gridLayout_24")
    self.label_23 = QtWidgets.QLabel(self.frame_11)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_23.sizePolicy().hasHeightForWidth())
    self.label_23.setSizePolicy(sizePolicy)
    self.label_23.setObjectName("label_23")
    self.gridLayout_24.addWidget(self.label_23, 0, 0, 1, 1)
    self.lvec_spinBox = QtWidgets.QDoubleSpinBox(self.frame_11)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.lvec_spinBox.sizePolicy().hasHeightForWidth())
    self.lvec_spinBox.setSizePolicy(sizePolicy)
    self.lvec_spinBox.setObjectName("lvec_spinBox")
    self.gridLayout_24.addWidget(self.lvec_spinBox, 0, 1, 1, 1)
    self.gridLayout_16.addWidget(self.frame_11, 0, 2, 1, 1)
    self.gridLayout_19.addWidget(self.frame_3, 1, 0, 1, 1)
    self.frame = QtWidgets.QFrame(self.groupBox_2)
    self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame.setObjectName("frame")
    self.gridLayout_29 = QtWidgets.QGridLayout(self.frame)
    self.gridLayout_29.setContentsMargins(0, 0, 0, 0)
    self.gridLayout_29.setObjectName("gridLayout_29")
    self.label_15 = QtWidgets.QLabel(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_15.sizePolicy().hasHeightForWidth())
    self.label_15.setSizePolicy(sizePolicy)
    self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_15.setObjectName("label_15")
    self.gridLayout_29.addWidget(self.label_15, 0, 3, 1, 1)
    self.hkldist_spinBox = QtWidgets.QDoubleSpinBox(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.hkldist_spinBox.sizePolicy().hasHeightForWidth())
    self.hkldist_spinBox.setSizePolicy(sizePolicy)
    self.hkldist_spinBox.setObjectName("hkldist_spinBox")
    self.gridLayout_29.addWidget(self.hkldist_spinBox, 0, 1, 1, 1)
    self.clipwidth_spinBox = QtWidgets.QDoubleSpinBox(self.frame)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.clipwidth_spinBox.sizePolicy().hasHeightForWidth())
    self.clipwidth_spinBox.setSizePolicy(sizePolicy)
    self.clipwidth_spinBox.setObjectName("clipwidth_spinBox")
    self.gridLayout_29.addWidget(self.clipwidth_spinBox, 0, 4, 1, 1)
    self.label_14 = QtWidgets.QLabel(self.frame)
    self.label_14.setObjectName("label_14")
    self.gridLayout_29.addWidget(self.label_14, 0, 0, 1, 1)
    self.gridLayout_19.addWidget(self.frame, 2, 0, 1, 1)
    self.gridLayout_7.addWidget(self.groupBox_2, 1, 0, 1, 3)
    self.gridLayout_9.addWidget(self.ClipPlaneChkGroupBox, 1, 0, 1, 2)
    self.showsliceGroupCheckbox = QtWidgets.QGroupBox(self.tab_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.showsliceGroupCheckbox.sizePolicy().hasHeightForWidth())
    self.showsliceGroupCheckbox.setSizePolicy(sizePolicy)
    self.showsliceGroupCheckbox.setCheckable(True)
    self.showsliceGroupCheckbox.setChecked(True)
    self.showsliceGroupCheckbox.setObjectName("showsliceGroupCheckbox")
    self.gridLayout_6 = QtWidgets.QGridLayout(self.showsliceGroupCheckbox)
    self.gridLayout_6.setContentsMargins(3, 3, 3, 3)
    self.gridLayout_6.setObjectName("gridLayout_6")
    self.SliceLabelComboBox = QtWidgets.QComboBox(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.SliceLabelComboBox.sizePolicy().hasHeightForWidth())
    self.SliceLabelComboBox.setSizePolicy(sizePolicy)
    self.SliceLabelComboBox.setObjectName("SliceLabelComboBox")
    self.gridLayout_6.addWidget(self.SliceLabelComboBox, 0, 0, 1, 1)
    self.label_3 = QtWidgets.QLabel(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
    self.label_3.setSizePolicy(sizePolicy)
    self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_3.setIndent(1)
    self.label_3.setObjectName("label_3")
    self.gridLayout_6.addWidget(self.label_3, 0, 1, 1, 1)
    self.sliceindexspinBox = QtWidgets.QSpinBox(self.showsliceGroupCheckbox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.sliceindexspinBox.sizePolicy().hasHeightForWidth())
    self.sliceindexspinBox.setSizePolicy(sizePolicy)
    self.sliceindexspinBox.setObjectName("sliceindexspinBox")
    self.gridLayout_6.addWidget(self.sliceindexspinBox, 0, 2, 1, 1)
    self.gridLayout_9.addWidget(self.showsliceGroupCheckbox, 2, 0, 1, 2)
    self.functionTabWidget.addTab(self.tab_2, "")
    self.tab_3 = QtWidgets.QWidget()
    self.tab_3.setObjectName("tab_3")
    self.gridLayout_27 = QtWidgets.QGridLayout(self.tab_3)
    self.gridLayout_27.setObjectName("gridLayout_27")
    self.groupBox_3 = QtWidgets.QGroupBox(self.tab_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
    self.groupBox_3.setSizePolicy(sizePolicy)
    self.groupBox_3.setCheckable(False)
    self.groupBox_3.setObjectName("groupBox_3")
    self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_3)
    self.gridLayout_10.setObjectName("gridLayout_10")
    self.label_7 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
    self.label_7.setSizePolicy(sizePolicy)
    self.label_7.setTextFormat(QtCore.Qt.AutoText)
    self.label_7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
    self.label_7.setWordWrap(True)
    self.label_7.setObjectName("label_7")
    self.gridLayout_10.addWidget(self.label_7, 0, 0, 1, 4)
    self.ManualPowerScalecheckbox = QtWidgets.QCheckBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ManualPowerScalecheckbox.sizePolicy().hasHeightForWidth())
    self.ManualPowerScalecheckbox.setSizePolicy(sizePolicy)
    self.ManualPowerScalecheckbox.setToolTip("")
    self.ManualPowerScalecheckbox.setObjectName("ManualPowerScalecheckbox")
    self.gridLayout_10.addWidget(self.ManualPowerScalecheckbox, 1, 0, 1, 2)
    self.label_10 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_10.sizePolicy().hasHeightForWidth())
    self.label_10.setSizePolicy(sizePolicy)
    self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
    self.label_10.setIndent(1)
    self.label_10.setObjectName("label_10")
    self.gridLayout_10.addWidget(self.label_10, 1, 2, 1, 1)
    self.power_scale_spinBox = QtWidgets.QDoubleSpinBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.power_scale_spinBox.sizePolicy().hasHeightForWidth())
    self.power_scale_spinBox.setSizePolicy(sizePolicy)
    self.power_scale_spinBox.setToolTip("")
    self.power_scale_spinBox.setMaximum(1.0)
    self.power_scale_spinBox.setSingleStep(0.05)
    self.power_scale_spinBox.setProperty("value", 0.35)
    self.power_scale_spinBox.setObjectName("power_scale_spinBox")
    self.gridLayout_10.addWidget(self.power_scale_spinBox, 1, 3, 1, 1)
    self.label_11 = QtWidgets.QLabel(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_11.sizePolicy().hasHeightForWidth())
    self.label_11.setSizePolicy(sizePolicy)
    self.label_11.setObjectName("label_11")
    self.gridLayout_10.addWidget(self.label_11, 2, 0, 1, 1)
    self.radii_scale_spinBox = QtWidgets.QDoubleSpinBox(self.groupBox_3)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.radii_scale_spinBox.sizePolicy().hasHeightForWidth())
    self.radii_scale_spinBox.setSizePolicy(sizePolicy)
    self.radii_scale_spinBox.setMaximum(5.0)
    self.radii_scale_spinBox.setSingleStep(0.1)
    self.radii_scale_spinBox.setObjectName("radii_scale_spinBox")
    self.gridLayout_10.addWidget(self.radii_scale_spinBox, 2, 1, 1, 1)
    self.gridLayout_27.addWidget(self.groupBox_3, 0, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_3, "")
    self.tab_4 = QtWidgets.QWidget()
    self.tab_4.setObjectName("tab_4")
    self.gridLayout_13 = QtWidgets.QGridLayout(self.tab_4)
    self.gridLayout_13.setObjectName("gridLayout_13")
    self.groupBox_5 = QtWidgets.QGroupBox(self.tab_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
    self.groupBox_5.setSizePolicy(sizePolicy)
    self.groupBox_5.setObjectName("groupBox_5")
    self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_5)
    self.gridLayout_11.setObjectName("gridLayout_11")
    self.BinDataComboBox = QtWidgets.QComboBox(self.groupBox_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(2)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.BinDataComboBox.sizePolicy().hasHeightForWidth())
    self.BinDataComboBox.setSizePolicy(sizePolicy)
    self.BinDataComboBox.setObjectName("BinDataComboBox")
    self.gridLayout_11.addWidget(self.BinDataComboBox, 0, 0, 1, 1)
    self.label_12 = QtWidgets.QLabel(self.groupBox_5)
    self.label_12.setObjectName("label_12")
    self.gridLayout_11.addWidget(self.label_12, 0, 1, 1, 1)
    self.Nbins_spinBox = QtWidgets.QSpinBox(self.groupBox_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.Nbins_spinBox.sizePolicy().hasHeightForWidth())
    self.Nbins_spinBox.setSizePolicy(sizePolicy)
    self.Nbins_spinBox.setMinimum(1)
    self.Nbins_spinBox.setMaximum(20)
    self.Nbins_spinBox.setObjectName("Nbins_spinBox")
    self.gridLayout_11.addWidget(self.Nbins_spinBox, 0, 2, 1, 1)
    self.gridLayout_13.addWidget(self.groupBox_5, 0, 0, 1, 1)
    self.widget_6 = QtWidgets.QWidget(self.tab_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.widget_6.sizePolicy().hasHeightForWidth())
    self.widget_6.setSizePolicy(sizePolicy)
    self.widget_6.setObjectName("widget_6")
    self.gridLayout_12 = QtWidgets.QGridLayout(self.widget_6)
    self.gridLayout_12.setContentsMargins(3, 3, 3, 3)
    self.gridLayout_12.setObjectName("gridLayout_12")
    self.OpaqueAllCheckbox = QtWidgets.QCheckBox(self.widget_6)
    self.OpaqueAllCheckbox.setObjectName("OpaqueAllCheckbox")
    self.gridLayout_12.addWidget(self.OpaqueAllCheckbox, 0, 0, 1, 1)
    self.binstable = QtWidgets.QTableWidget(self.widget_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(1)
    sizePolicy.setHeightForWidth(self.binstable.sizePolicy().hasHeightForWidth())
    self.binstable.setSizePolicy(sizePolicy)
    self.binstable.setRowCount(5)
    self.binstable.setColumnCount(4)
    self.binstable.setObjectName("binstable")
    self.binstable.setColumnCount(4)
    self.binstable.setRowCount(5)
    self.gridLayout_12.addWidget(self.binstable, 1, 0, 1, 1)
    self.gridLayout_13.addWidget(self.widget_6, 1, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_4, "")
    self.tab_5 = QtWidgets.QWidget()
    self.tab_5.setObjectName("tab_5")
    self.gridLayout_8 = QtWidgets.QGridLayout(self.tab_5)
    self.gridLayout_8.setObjectName("gridLayout_8")
    self.groupBox_6 = QtWidgets.QGroupBox(self.tab_5)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
    self.groupBox_6.setSizePolicy(sizePolicy)
    self.groupBox_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
    self.groupBox_6.setObjectName("groupBox_6")
    self.gridLayout_21 = QtWidgets.QGridLayout(self.groupBox_6)
    self.gridLayout_21.setObjectName("gridLayout_21")
    self.DrawReciprocUnitCellBox = QtWidgets.QGroupBox(self.groupBox_6)
    self.DrawReciprocUnitCellBox.setCheckable(True)
    self.DrawReciprocUnitCellBox.setChecked(False)
    self.DrawReciprocUnitCellBox.setObjectName("DrawReciprocUnitCellBox")
    self.gridLayout_25 = QtWidgets.QGridLayout(self.DrawReciprocUnitCellBox)
    self.gridLayout_25.setObjectName("gridLayout_25")
    self.label_2 = QtWidgets.QLabel(self.DrawReciprocUnitCellBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
    self.label_2.setSizePolicy(sizePolicy)
    self.label_2.setWordWrap(True)
    self.label_2.setObjectName("label_2")
    self.gridLayout_25.addWidget(self.label_2, 0, 0, 1, 1)
    self.reciprocunitcellslider = QtWidgets.QSlider(self.DrawReciprocUnitCellBox)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.reciprocunitcellslider.sizePolicy().hasHeightForWidth())
    self.reciprocunitcellslider.setSizePolicy(sizePolicy)
    self.reciprocunitcellslider.setMinimum(0)
    self.reciprocunitcellslider.setMaximum(100)
    self.reciprocunitcellslider.setOrientation(QtCore.Qt.Horizontal)
    self.reciprocunitcellslider.setObjectName("reciprocunitcellslider")
    self.gridLayout_25.addWidget(self.reciprocunitcellslider, 0, 1, 1, 1)
    self.label_4 = QtWidgets.QLabel(self.DrawReciprocUnitCellBox)
    self.label_4.setWordWrap(True)
    self.label_4.setObjectName("label_4")
    self.gridLayout_25.addWidget(self.label_4, 0, 2, 1, 1)
    self.gridLayout_21.addWidget(self.DrawReciprocUnitCellBox, 1, 0, 1, 1)
    self.DrawRealUnitCellBox = QtWidgets.QGroupBox(self.groupBox_6)
    self.DrawRealUnitCellBox.setCheckable(True)
    self.DrawRealUnitCellBox.setChecked(False)
    self.DrawRealUnitCellBox.setObjectName("DrawRealUnitCellBox")
    self.gridLayout_26 = QtWidgets.QGridLayout(self.DrawRealUnitCellBox)
    self.gridLayout_26.setObjectName("gridLayout_26")
    self.label_6 = QtWidgets.QLabel(self.DrawRealUnitCellBox)
    self.label_6.setWordWrap(True)
    self.label_6.setObjectName("label_6")
    self.gridLayout_26.addWidget(self.label_6, 0, 0, 1, 1)
    self.unitcellslider = QtWidgets.QSlider(self.DrawRealUnitCellBox)
    self.unitcellslider.setMinimum(0)
    self.unitcellslider.setMaximum(100)
    self.unitcellslider.setOrientation(QtCore.Qt.Horizontal)
    self.unitcellslider.setObjectName("unitcellslider")
    self.gridLayout_26.addWidget(self.unitcellslider, 0, 1, 1, 1)
    self.label_5 = QtWidgets.QLabel(self.DrawRealUnitCellBox)
    self.label_5.setWordWrap(True)
    self.label_5.setObjectName("label_5")
    self.gridLayout_26.addWidget(self.label_5, 0, 2, 1, 1)
    self.gridLayout_21.addWidget(self.DrawRealUnitCellBox, 2, 0, 1, 1)
    self.frame_4 = QtWidgets.QFrame(self.groupBox_6)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.frame_4.sizePolicy().hasHeightForWidth())
    self.frame_4.setSizePolicy(sizePolicy)
    self.frame_4.setFrameShape(QtWidgets.QFrame.StyledPanel)
    self.frame_4.setFrameShadow(QtWidgets.QFrame.Raised)
    self.frame_4.setObjectName("frame_4")
    self.gridLayout_14 = QtWidgets.QGridLayout(self.frame_4)
    self.gridLayout_14.setObjectName("gridLayout_14")
    self.ResetViewBtn = QtWidgets.QPushButton(self.frame_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.ResetViewBtn.sizePolicy().hasHeightForWidth())
    self.ResetViewBtn.setSizePolicy(sizePolicy)
    self.ResetViewBtn.setObjectName("ResetViewBtn")
    self.gridLayout_14.addWidget(self.ResetViewBtn, 0, 0, 1, 1)
    self.SaveImageBtn = QtWidgets.QPushButton(self.frame_4)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.SaveImageBtn.sizePolicy().hasHeightForWidth())
    self.SaveImageBtn.setSizePolicy(sizePolicy)
    self.SaveImageBtn.setObjectName("SaveImageBtn")
    self.gridLayout_14.addWidget(self.SaveImageBtn, 0, 1, 1, 1)
    self.gridLayout_21.addWidget(self.frame_4, 0, 0, 1, 1)
    self.gridLayout_8.addWidget(self.groupBox_6, 0, 0, 1, 1)
    self.functionTabWidget.addTab(self.tab_5, "")
    self.textInfo = QtWidgets.QPlainTextEdit(self.splitter_2)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    sizePolicy.setHorizontalStretch(0)
    sizePolicy.setVerticalStretch(2)
    sizePolicy.setHeightForWidth(self.textInfo.sizePolicy().hasHeightForWidth())
    self.textInfo.setSizePolicy(sizePolicy)
    self.textInfo.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.textInfo.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
    self.textInfo.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
    self.textInfo.setObjectName("textInfo")
    self.gridLayout_4.addWidget(self.splitter_2, 0, 0, 1, 1)
    self.BrowserBox = QWebEngineView(self.splitter)
    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
    sizePolicy.setHorizontalStretch(1)
    sizePolicy.setVerticalStretch(0)
    sizePolicy.setHeightForWidth(self.BrowserBox.sizePolicy().hasHeightForWidth())
    self.BrowserBox.setSizePolicy(sizePolicy)
    self.BrowserBox.setObjectName("BrowserBox")
    self.gridLayout.addWidget(self.splitter, 0, 0, 1, 1)
    self.gridLayout_2.addWidget(self.widget, 0, 0, 1, 1)
    MainWindow.setCentralWidget(self.centralwidget)
    self.menubar = QtWidgets.QMenuBar(MainWindow)
    self.menubar.setGeometry(QtCore.QRect(0, 0, 1066, 18))
    self.menubar.setObjectName("menubar")
    self.menuFile = QtWidgets.QMenu(self.menubar)
    self.menuFile.setObjectName("menuFile")
    MainWindow.setMenuBar(self.menubar)
    self.statusBar = QtWidgets.QStatusBar(MainWindow)
    self.statusBar.setObjectName("statusBar")
    MainWindow.setStatusBar(self.statusBar)
    self.actionOpen_reflection_file = QtWidgets.QAction(MainWindow)
    self.actionOpen_reflection_file.setObjectName("actionOpen_reflection_file")
    self.actionSettings = QtWidgets.QAction(MainWindow)
    self.actionSettings.setObjectName("actionSettings")
    self.actiondebug = QtWidgets.QAction(MainWindow)
    self.actiondebug.setObjectName("actiondebug")
    self.actionExit = QtWidgets.QAction(MainWindow)
    self.actionExit.setObjectName("actionExit")
    self.actionSave_reflection_file = QtWidgets.QAction(MainWindow)
    self.actionSave_reflection_file.setObjectName("actionSave_reflection_file")
    self.menuFile.addAction(self.actionOpen_reflection_file)
    self.menuFile.addAction(self.actionSave_reflection_file)
    self.menuFile.addAction(self.actionSettings)
    self.menuFile.addAction(self.actiondebug)
    self.menuFile.addAction(self.actionExit)
    self.menubar.addAction(self.menuFile.menuAction())

    self.retranslateUi(MainWindow)
    self.functionTabWidget.setCurrentIndex(0)
    QtCore.QMetaObject.connectSlotsByName(MainWindow)

  def retranslateUi(self, MainWindow):
    MainWindow.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "MainWindow", None, -1))
    self.label.setText(QtWidgets.QApplication.translate("MainWindow", "Display a data set with a double-click. Right-click table for more options.", None, -1))
    self.ExpandReflsGroupBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Expand reflections", None, -1))
    self.expandP1checkbox.setText(QtWidgets.QApplication.translate("MainWindow", "Expand to P1", None, -1))
    self.expandAnomalouscheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Friedel pairs", None, -1))
    self.SpacegroupLabel.setText(QtWidgets.QApplication.translate("MainWindow", "Space Subgroups", None, -1))
    self.sysabsentcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Systematic Absences", None, -1))
    self.missingcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show Missing Reflections", None, -1))
    self.onlymissingcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Only", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab), QtWidgets.QApplication.translate("MainWindow", "Expansion", None, -1))
    self.SliceReflectionsBox.setText(QtWidgets.QApplication.translate("MainWindow", "Slice Reflections", None, -1))
    self.fixedorientcheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Fix orientation but allow zoom and translation", None, -1))
    self.ClipPlaneChkGroupBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "with a clip plane oriented", None, -1))
    self.clipNormalBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Perpendicular to vector below", None, -1))
    self.clipParallelBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Parallel to vector below", None, -1))
    self.groupBox_2.setTitle(QtWidgets.QApplication.translate("MainWindow", "Vector Components (r1, r2, r3) are specified", None, -1))
    self.recipvecBtn.setText(QtWidgets.QApplication.translate("MainWindow", "in reciprocal unit cell coordinates", None, -1))
    self.clipTNCSBtn.setText(QtWidgets.QApplication.translate("MainWindow", "as tNCS vector", None, -1))
    self.realspacevecBtn.setText(QtWidgets.QApplication.translate("MainWindow", "in real space unit cell coordinates", None, -1))
    self.rotavecangle_labeltxt.setText(QtWidgets.QApplication.translate("MainWindow", "Angle rotated around vector ", None, -1))
    self.label_21.setText(QtWidgets.QApplication.translate("MainWindow", "r1:", None, -1))
    self.label_22.setText(QtWidgets.QApplication.translate("MainWindow", "r2:", None, -1))
    self.label_23.setText(QtWidgets.QApplication.translate("MainWindow", "r3:", None, -1))
    self.label_15.setText(QtWidgets.QApplication.translate("MainWindow", "Clip Plane Width:", None, -1))
    self.label_14.setText(QtWidgets.QApplication.translate("MainWindow", "Distance from origin:", None, -1))
    self.showsliceGroupCheckbox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Explicitly with a plane at", None, -1))
    self.label_3.setText(QtWidgets.QApplication.translate("MainWindow", "equal to", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_2), QtWidgets.QApplication.translate("MainWindow", "Slicing", None, -1))
    self.groupBox_3.setTitle(QtWidgets.QApplication.translate("MainWindow", "Radii size of HKL spheres", None, -1))
    self.label_7.setText(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Tick the &quot;User defined power scaling&quot; box for chosing a power factor. Untick this box for an automatically computed power factor. Automatic power factor will render reflections so that the reflection with the smallest data value is rendered about 10 times smaller than the reflection with the largest data value. <br/>Entering a power factor of 1 means the size of a sphere for each reflection is proportional to its data value. For a typical data set of intensities with values differing by some orders of magnitude only the strongest reflections would then be visible. Entering a power factor of 0 means all spheres of reflections will have the same size irrespective of their data values. This means even very weak reflections would then be visible. </p></body></html>", None, -1))
    self.ManualPowerScalecheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "User defined power scaling", None, -1))
    self.label_10.setText(QtWidgets.QApplication.translate("MainWindow", "Power scale factor:", None, -1))
    self.label_11.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>A linear scale factor of 1 means that if two reflectiions with the largest data values happen to be next to each other their spheres will touch but not overlap.</p></body></html>", None, -1))
    self.label_11.setText(QtWidgets.QApplication.translate("MainWindow", "Linear scale factor", None, -1))
    self.radii_scale_spinBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>A linear scale factor of 1 means that if two reflectiions with the largest data values happen to be next to each other their spheres will touch but not overlap.</p></body></html>", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_3), QtWidgets.QApplication.translate("MainWindow", "Sizing", None, -1))
    self.groupBox_5.setTitle(QtWidgets.QApplication.translate("MainWindow", "Bin according to", None, -1))
    self.BinDataComboBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Order the curently displayed data set in bins according to the values of the selected data set. Reflections in the currently displayed data set not matching any reflections in the selected data set will be put in a bin labelled with &quot;NaN&quot;.</p></body></html>", None, -1))
    self.label_12.setText(QtWidgets.QApplication.translate("MainWindow", "Number of bins:", None, -1))
    self.Nbins_spinBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Select the number of bins for ordering the currently displayed data set. More than about 10-15 bins will slow down the HKLviewer when working with large data sets.</p></body></html>", None, -1))
    self.OpaqueAllCheckbox.setText(QtWidgets.QApplication.translate("MainWindow", "Show/hide all data", None, -1))
    self.binstable.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Change the visibility of selected bins either by ticking or unticking the check boxes or by entering an opacity value between 0 and 1</p></body></html>", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_4), QtWidgets.QApplication.translate("MainWindow", "Binning", None, -1))
    self.groupBox_6.setTitle(QtWidgets.QApplication.translate("MainWindow", "Geometry", None, -1))
    self.DrawReciprocUnitCellBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>The reciprocal unit cell spanning a*,b*,c* from (0,0,0) to (1,1,1) is located in the centre of the displayed reflections. and is therefore quite small. Adjust the slider to scale its outline to become compatible with the sphere of displayed reflections.</p></body></html>", None, -1))
    self.DrawReciprocUnitCellBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Show Reciprocal Unit Cell", None, -1))
    self.label_2.setText(QtWidgets.QApplication.translate("MainWindow", "True to Scale of HKL grid", None, -1))
    self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "Close to radius of sphere of reflections", None, -1))
    self.DrawRealUnitCellBox.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>The real space unit cell is often much larger than the sphere of displayed reflections because the reciprocal unit cell size is inversely proportional to the size of the real space unit cell. Adjust the slider to scale its outline to become compatible with the sphere of displayed reflections.</p></body></html>", None, -1))
    self.DrawRealUnitCellBox.setTitle(QtWidgets.QApplication.translate("MainWindow", "Show Real Space Unit Cell", None, -1))
    self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "True to Scale of HKL grid", None, -1))
    self.label_5.setText(QtWidgets.QApplication.translate("MainWindow", "Close to radius of sphere of reflections", None, -1))
    self.ResetViewBtn.setToolTip(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p>Place the displayed reflections in the centre of the view.</p></body></html>", None, -1))
    self.ResetViewBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Reset View", None, -1))
    self.SaveImageBtn.setText(QtWidgets.QApplication.translate("MainWindow", "Save Image", None, -1))
    self.functionTabWidget.setTabText(self.functionTabWidget.indexOf(self.tab_5), QtWidgets.QApplication.translate("MainWindow", "Extras", None, -1))
    self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
    self.actionOpen_reflection_file.setText(QtWidgets.QApplication.translate("MainWindow", "Open reflection file...", None, -1))
    self.actionSettings.setText(QtWidgets.QApplication.translate("MainWindow", "Settings", None, -1))
    self.actiondebug.setText(QtWidgets.QApplication.translate("MainWindow", "Debug", None, -1))
    self.actionExit.setText(QtWidgets.QApplication.translate("MainWindow", "Exit", None, -1))
    self.actionSave_reflection_file.setText(QtWidgets.QApplication.translate("MainWindow", "Save reflection file", None, -1))




from PySide2.QtWebEngineWidgets import QWebEngineView
try: # if invoked by cctbx.python or some such
  from crys3d.hklview.helpers import HeaderDataTableWidget
except Exception as e: # if invoked by a generic python that doesn't know cctbx modules
  from helpers import HeaderDataTableWidget
